## monitor.m - Checks the status of the cluster 
## Author: Grant Ingram
## Notes: Before running check ALL file destinations
##        Also ssh keys need to be set up to allow pasword free login 

# Copyright 2009,2010  Richard James MacPherson, Grant Ingram and Jamie McIntosh
# This file is part of PEW.

#    PEW is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    PEW is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with PEW.  If not, see <http://www.gnu.org/licenses/>.

#####################
# Revision History - Jamie McIntosh
# 1.01 - ssh connection now possible from outside of university network
# 1.02 - Survivors of each generation are copied into the next Generation
#        filestore so that repeat testing is removed and script modified to
#        skip these chromosomes in CFD analysis
# 1.03 - Reverted to 1.01 - skipping affects mutation
# 1.04 - Monitor adapted to scan batched jobs individually
# 1.06b - THIS VERSION HAS NO FV SUPPORT AND RUNS ON VEGA
#####################
# Known Issues
#
#####################

function monitor() 

global iga; global base_dir; global cluster_dir; global job_id;
global popsize; global user_id; global keep;

disp("Monitoring the job status...")
tic();  # Start timer for the generation
donejobs = 0; 
while (1)
  printf("Hamilton Jobs:");
  printf(" %d",job_id(:,1)); % Display the jobs sent to hamilton
  printf("\n");
  donejobs = 0; errornum = 0; # reset counters
  for chrom=1:1:(popsize) % Cycles through each chromosome
    printf("Chromosome %d:\t",chrom);
    chrom_dir = sprintf("%sGeneration_%d/chr_%d/",cluster_dir,iga,chrom);
    jobfile_gambit = ['gambit.out'];
    jobfile_fluent = ['fluent.out'];

    # Check Meshing Progress
    [status,output] = system(sprintf("ssh %s@vega3.dur.ac.uk 'test -e %s%s && echo 10'"
		     ,user_id,chrom_dir,jobfile_gambit),1);

    if (output > 0 ) # The file exists so running or done

      # Check for 'successfully' in the outputfile
      [status,output1] = system(sprintf(
			"ssh %s@vega3.dur.ac.uk 'less %s%s | grep successfully | wc -l'"
			,user_id,chrom_dir,jobfile_gambit));
      
      # Check for 'EQUISIZE' in the outputfile
      [status,output2] = system(sprintf(
			"ssh %s@vega3.dur.ac.uk 'less %s%s | grep EQUISIZE | wc -l'"
			,user_id,chrom_dir,jobfile_gambit));
      if (str2num(output1) > 0)
	# 'successfully' in the output file so mesh exported successfully
	printf("Mesh: Done\t");
      elseif (str2num(output2) > 0)
	# 'EQUISIZE' in the output file so mesh failed
	printf("Mesh: Failed\t"); 
      else
	# File exists and not finished so we must still be running...
	printf("Mesh: Running\t");
      endif
    else 
      # No file so the system must still be waiting for a slot...
      printf("Mesh: Waiting\t");
    endif

    # Check CFD Progress
    [status,output] = system(sprintf("ssh %s@vega3.dur.ac.uk 'test -e %s%s && echo 10'"
		     ,user_id,chrom_dir,jobfile_fluent),1);

    if (output > 0 ) # The file exists so running, halted or done

      # Check for results file 
      [status,output1] = system(sprintf("ssh %s@vega3.dur.ac.uk 'test -e %s%s && echo 10'"
				       ,user_id,chrom_dir,"res"),1);

      # Check for Halting in the outputfile
      [status,output2] = system(sprintf(
			"ssh %s@vega3.dur.ac.uk 'less %s%s | grep Halting | wc -l'"
			,user_id,chrom_dir,jobfile_fluent));
      if (output1 > 0)
	# Results files exist so job is done....
	disp(sprintf("CFD: Done"));
	donejobs = donejobs + 1;
      elseif (str2num(output2) > 0)
	# Halted in the output file so job is halted
	disp(sprintf("CFD: Halted"));
	donejobs = donejobs + 1;
	errornum = errornum +1;
      else
	# File exists, no results and not halted so we must still be running...
	disp(sprintf("CFD: Running"));
      endif
    else 
      # No file so the system must still be waiting for a slot...
      disp(sprintf("CFD: Waiting"));
    endif

  endfor

  if (donejobs == popsize) 
    break
  else
    disp("*** Waiting 120 seconds ***");
    pause(120); 
  endif

endwhile

disp('All chromosomes done');
elapsed_time = toc();
printf("\nGeneration %d took %f hours",iga,elapsed_time/3600);

endfunction
