#!/usr/bin/octave -q
# oegularise - interpolate irregular data onto a regular grid
# Copyright 2009,2010  Richard James MacPherson, Grant Ingram and Jamie McIntosh
# This file is part of PEW.

#    PEW is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    PEW is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with PEW.  If not, see <http://www.gnu.org/licenses/>.

# disp("oegularise - interpolate onto regular grid");
margin = 1; # 1mm margin inside the data range
npoints = 100;  # Use 100x100 grid to interpolate onto
step = 5;
if (nargin < 2)
  printf("\nusage: oegularise input output");
  exit(5);
endif
arg_list = argv ();

input = arg_list{1};  output = arg_list{2};
data = dlmread(input,'\t',1,0); #Uses octave-forge io function

axial = data(1,1);
x = data(:,2); y = data(:,3); # just to make it easier to follow

xmin = min(x)+margin; 
xmax = max(x)-margin;
ymin = min(y)+margin;

if (max(y) < 185)
  printf("\nError: dataset is below midspan");
  ymax = max(y)-margin;
else
  ymax = 185;
endif

## If xi and yi are not of the same length they get meshgridded into
## something else.... we don't want that so make sure they are of the
## same length.... there is a bug in griddata which means that if you
## send it arrays of the same length it barfs.. ummm... Solve this by
## adding one more row in the x-direction and then using the arrays
## returned from griddata for x and y

xi = linspace(xmin,xmax,npoints+1);
yi = linspace(ymin,ymax,npoints);

# Step through each column in the file and interpolate
# first three lines (1,2,3) are geometry...
# Need new XI and YI due to the griddata bug above
for var = 4:columns(data)
  [XI,YI,ZI(:,:,var)] = griddata (x, y, data(:,var), xi, yi,"linear");
endfor

## Quick test to make sure that the thing works.... 
## mesh (XI, YI,ZI(:,:,6)); print("test.eps","-depsc2");

# Output data
fout = fopen(output, 'w');
# Output header information
fprintf(fout,"# Output produced by oegularise script\n")
fprintf(fout,"# Output produced by oegularise script\n")
fprintf(fout,"# Output produced by oegularise script\n")
fprintf(fout,"5\t%d\t1\t%d\t%d\n"
	,columns(XI)*rows(YI),columns(XI),rows(YI));
fprintf(fout,"axial\ttang\tradial\tturret\tpitch\tyaw\tptot\tpstat\n");

## It is probably clear here that your graps of rows and columns in the
## data is sketchy at best, but this does at least appear to work..

for i = 1:columns(XI)
  for j = 1:rows(YI)
    fprintf(fout,"%f\t%f\t%f\t",axial,XI(j,i),YI(j,i));
       for k=4:columns(data)
	 fprintf(fout,"%5.5f\t", ZI(j,i,k));
       endfor
       fprintf(fout,"\n");
    endfor		   
endfor
   
fclose(fout);
